/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.internal;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

/**
 * @author Henrik Kaipe
 */
public class CallbackContextFactory {

    public static CallbackContext forClassLoader(ClassLoader loader)
    throws Exception {
        final Class remoteClass = loader
                .loadClass(CallbackContextImpl.class.getName());
        Constructor c = remoteClass.getDeclaredConstructors()[0];
        c.setAccessible(true);
        final Object ctx = c.newInstance(null);
        if (CallbackContextImpl.class == remoteClass) {
            return (CallbackContext) ctx;
        }

        final CallbackContext narrowedRemote = (CallbackContext)
                Proxy.newProxyInstance(
                CallbackContextImpl.class.getClassLoader(),
                new Class[] {CallbackContext.class},
                new InvocationHandler() {
            public Object invoke(Object proxy, Method m, Object[] args)
            throws Throwable {
                Method remoteMethod = remoteClass.getMethod(
                        m.getName(), m.getParameterTypes());
                remoteMethod.setAccessible(true);
                return remoteMethod.invoke(ctx, args);
            }
        });

        return CallbackContextBridge.newInstance(loader, narrowedRemote);
    }
}
