/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit3.support;

import org.callbackparams.junit3.CallbackTestCase;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import org.callbackparams.AdaptiveRule;
import org.callbackparams.internal.CallbackRebyteInfo;
import org.callbackparams.internal.GeneralReloadFilter;

/**
 * @author Henrik Kaipe
 */
public abstract class RebyteInfo implements CallbackRebyteInfo {

    private static final GeneralReloadFilter reloadFilter =
            new GeneralReloadFilter(null);

    public boolean isCallbackInjectField(Field f) {
        return f.getType().isInterface()
                && false == Modifier.isStatic(f.getModifiers())
                && false == Modifier.isFinal(f.getModifiers());
    }

    public boolean isAdaptiveRuleField(Field f) {
        return AdaptiveRule.class.isAssignableFrom(f.getType());
    }

    public boolean isCallbackProxiedMethod(Method m) {
        return isTestMethod(m)
                && 0 < m.getParameterTypes().length;
    }

    public boolean isTestMethod(Method m) {
        if (false == Modifier.isPublic(m.getModifiers())
                || false == m.getName().startsWith("test")) {
            return false;
        }
        final Class[] params = m.getParameterTypes();
        for (int i = 0 ; i < params.length ; ++i) {
            if (false == params[i].isInterface()) {
                return false;
            }
        }
        return true;
    }

    public boolean isExplicitTopClass(Class clazz) {
        Class superClass = clazz.getSuperclass();
        return CallbackTestCase.class == superClass ||
                false == reloadFilter.needsToBeReloaded(superClass.getName());
    }
    public Class adaptiveRuleSuperClass() {
        return null;
    }
}
