/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4;

import org.junit.runner.Description;
import org.junit.runner.Runner;
import org.junit.runner.notification.RunNotifier;

/**
 * The recommended strategy for JUnit-integration is to use this Runner
 * implementation, by annotating the test-class with ...
 * <pre>
 * &#64;org.junit.runner.RunWith(CallbackParamsRunner.class)
 * </pre>
 * ... and possibly complement with some combine-configuration ...
 * <pre>
 * {@link org.callbackparams.combine.annotation.CombineConfig @CombineConfig}(maxCount=100, strategy={@link org.callbackparams.combine.CombineCompletely CombineCompletely}.class)
 * </pre>
 * This will work fine even if the test-class uses the JUnit-3.x API, instead of
 * the JUnit-4.x annotations @Test, @Before etc. CallbackParamsRunner will
 * (1) autodetect which JUnit API-version is in use, (2) modify the test-class'
 * bytecode accordingly and (3) pick an appropriate Runner-implementation to
 * take care of the actual test-runs.<br/><br/>
 * Please note that it is possible to specify an arbitrary Runner-implementation
 * class by using the {@link WrappedRunner @WrappedRunner} annotation. This will
 * work for just about any Runner as long as its normal usage resembles the
 * default JUnit runner's usage of the annotations @Test, @Before, @After.
 *
 * @author Henrik Kaipe
 */
public class CallbackParamsRunner extends Runner {

    /**
     * The class CallbackParamsRunner is nothing but a thin wrapper for the
     * actual runner implementation that will be kept inside this instance
     * of an appropriate RunWrapper implementation class.
     * The abstract class RunWrapper has two implementations, one that makes
     * byte-code modifications to support the JUnit-3.x API and one that honors
     * the JUnit-4.x API. These implementations do little more than taking care
     * of bytecode modifications and detection of which JUnit API-version is
     * used. The actual test-run will be handed over to the appropriate JUnit
     * Runner implementation or any other Runner implementation that has been
     * specified with the @WrappedRunner annotation.
     */
    private final RunWrapper runWrapper;

    public CallbackParamsRunner(Class testClass) throws ClassNotFoundException {
        this.runWrapper = RunWrapper.newInstance(testClass);
    }

    public Description getDescription() {
        return this.runWrapper.getDescription();
    }

    public void run(RunNotifier notifier) {
        this.runWrapper.run(notifier);
    }
}
