/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.callbackparams.junit4;

import org.callbackparams.internal.CallbackTestClassReloader;

/**
 * The annotation class {@link WrappedRunner} must not be reloaded or the
 * runner will have a problem. This should be fairly safe as the annotation
 * is unaware about everything but itself and the JUnit, which is not
 * reloaded either.
 * 
 * @author Henrik Kaipe
 */
class WrappedRunnnerReloadFilter
implements CallbackTestClassReloader.ReloadFilter {

    private static final String wrappedRunnerClassName =
            WrappedRunner.class.getName();

    private static final WrappedRunnnerReloadFilter instance =
            new WrappedRunnnerReloadFilter();

    public boolean needsToBeReloaded(String classNameToReload) {
        return false == classNameToReload.equals(wrappedRunnerClassName);
    }

    public static WrappedRunnnerReloadFilter getInstance() {
        return instance;
    }
}
